/*
   Sophos Anti-Virus for OS/2
   Procedure to do a fast track install / update / restore.

   Version 3.13  NRP   20001205

   Bodge check of diskette labels to take account of non-standard
   product IDs used for -AH release while parallel releases of -AH
   and "traditional" versions are being made.

   Copyright (c) 1998,2001 Sophos Plc, www.sophos.com
*/

   /*
      Shut up unwanted output from CMD.EXE unless running under PMREXX.
   */
   IF LEFT( TRACE(), 1 ) \= "?" THEN
      DO
         TRACE O
         "@ECHO OFF"
      END

   /*
      Get parameters without uppercasing strings!
   */
   PARSE ARG which userQualifiers

   CALL RxFuncAdd 'SysLoadFuncs', 'REXXUTIL', 'SysLoadFuncs'
   CALL SysLoadFuncs

   betaID = "OS"
   fullID = "OS"
   betaLabelID = "QO"   /* Temp for -AH release during parallel release period */
   fullLabelID = "AO"   /* Temp for -AH release during parallel release period */

   PARSE UPPER SOURCE . . sourceFile
   sourceDrive = FILESPEC( 'D', sourceFile )
   sourceDrivePath = sourceDrive || FILESPEC( 'P', sourceFile )

   /*
      Identical copies of this procedure are stored as
         INSTALL.CMD, UPDATE.CMD, RESTORE.CMD, etc.
      Determine which of these commands invoked this procedure,
         and set up indices accordingly.
   */
   actionName = LEFT( FILESPEC( 'N', sourceFile ), 4 )
   SELECT
      WHEN actionName = "CATA" THEN actionNumber = 0
      WHEN actionName = "INST" THEN actionNumber = 1
      WHEN actionName = "UPDA" THEN actionNumber = 2
      WHEN actionName = "REST" THEN actionNumber = 3
      WHEN actionName = "REMO" THEN
         DO
            actionNumber = 4
            actionName = "DELE"
         END
      WHEN actionName = "UNIN" THEN
         DO
            actionNumber = 4
            actionName = "DELE"
         END
   END

   /*
      Determine source medium volume label, and analyze it
      on the assumption that it is a standard Sophos label.
   */
   PARSE VALUE SysDriveInfo( sourceDrive ) WITH . . . VolID .
   PARSE VAR VolID ID 3 what 4 lang 5 medium 6 .
   medID = what || medium

   /*
      Determine what language we are using.
   */
/* TEMP */ lang = 'E'
   messageFile = sourceDrivePath || "OSI" || lang || ".INF"

   /*
      Look at the first UserQualifier on the command line.
      If it is an abbreviation of either "WORKSTATION" or "CENTRAL",
         store this as a user selection.
      If it is one of /? /H -? -H give a short help message and exit.
      If it is missing, give a short help message and demand that user
         stipulate one of "WORKSTATION" or "CENTRAL".
   */
   w = TRANSLATE( which )
   SELECT
      WHEN LEFT( w, 1 ) == "W" THEN
         which = 'W'
/*
      WHEN LEFT( w, 1 ) == "C" THEN
         which = 'C'
*/
      WHEN w == "/?" | w == "/H" | w == "-?" | w == "-H" THEN
         CALL GiveHelp messageFile
      OTHERWISE
         userQualifiers = which userQualifiers
         which = 'W'
   END

   /*
      Determine which catalog file is appropriate:
            the one for small diskettes
         or the one for "large" media or networks
      Search source medium for an appropriate catalog file.
   */
   IF ( medID == "I4" ) & ( (ID == betaLabelID) | (ID == fullLabelID) ) THEN
      DO
         /*
            We are coming from a diskette.
         */
         catalogMask = sourceDrivePath || fullID || "??M.ICF"
         e = SysFileTree( catalogMask, "catalog", "FO", "*--*-" )
         IF ( e > 0 ) | ( catalog.0 = 0 ) THEN
            DO
               catalogMask = sourceDrivePath || betaID || "??M.ICF"
               e = SysFileTree( catalogMask, "catalog", "FO", "*--*-" )
            END
      END
   ELSE
      DO
         /*
            We are not coming from a diskette, so it is a CD or
            network drive.  In the latter case, the volume label
            is not in the standard Sophos format, so we cannot
            use label information at all here.
         */
         catalogMask = sourceDrivePath || fullID || "??S.ICF"
         e = SysFileTree( catalogMask, "catalog", "FO", "*--*-" )
         IF ( e > 0 ) | ( catalog.0 = 0 ) THEN
            DO
               catalogMask = sourceDrivePath || betaID || "??S.ICF"
               e = SysFileTree( catalogMask, "catalog", "FO", "*--*-" )
            END
      END

   IF ( e > 0 ) | ( catalog.0 \= 1 ) THEN
      DO
         SAY "INTERNAL ERROR: Cannot find a unique catalog file" catalogMask
         /*
            Exit with CID-compliant errorlevel:
               Problem with product configuration / catalog file / package file
         */
         EXIT X2D( "0816" )
      END

   /*
      Look at our environment variables (set up in the system registry by
      the preceding installation phase), and determine whether to work on
      a CENTRAL installation or a WORKSTATION installation.
   */

   e1 = 0
   e2 = 0

/*
   IF which == 'C' THEN
      DO
         path = CheckAlreadyInstalled( "Central", actionNumber, messageFile )

         e1 = RunInstaller( actionName, sourceDrivePath, userQualifiers, catalog.1, "Sophos Anti-Virus for OS/2 Central Installation" )
         IF CheckErr( e1 ) THEN
            EXIT e1
      END
*/

   IF which == 'W' THEN
      DO
         path = CheckAlreadyInstalled( "Workstation", actionNumber, messageFile )
         IF path = "" THEN
            notRunning = 1                   /* not installed, so unlikely to be running */
         ELSE
            DO
               /*
                  Stop running SWEEP to avoid need for reboots of a file server running SWEEP.
                  This command returns errorlevel 1 if SWEEP was not running, 0 if it was running.
               */
               path || "SWEEP /CLOSE"
               notRunning = ( rc > 0 )
               CALL SysSleep 2
               IF \ notRunning THEN
                  DO 10 UNTIL rc > 0
                     path || "SWEEP /CLOSE"  /* make sure it's really gone */
                     CALL SysSleep 1         /* and then wait a little for luck */
                  END
            END

         e2 = RunInstaller( actionName, sourceDrivePath, userQualifiers, catalog.1, "Sophos Anti-Virus for OS/2 Workstation" )
         IF CheckErr( e2 ) THEN
            EXIT e2

         /*
            Start running SWEEP again if it was running previously.
            Do this by opening the startup object in order to
               get the installed default qualifiers
         */
         IF \ notRunning THEN
            CALL SysOpenObject "<SOPHOS_SAV_STARTUP>", 0, 0
      END

   EXIT MAX( e1, e2 )



/*
   Get a value from the system registry, and remove any garbage from it.
*/
GetFromRegistry: PROCEDURE
   /*
      Get parameters without uppercasing strings!
   */
   PARSE ARG key

   val = SysIni( "USER", "SOPHOS_Anti-Virus", key )
   val = STRIP( val, 'T', '0'X )       /* dock any trailing nulls */
   IF val == "ERROR:" THEN
      val = ""
   RETURN val



/*
   Run IBM Software Installer with suitable parameters.
*/
RunInstaller: PROCEDURE
   /*
      Get parameters without uppercasing strings!
   */
   PARSE ARG actionName, sourceDrivePath, userQualifiers, catalogFile, productName

   /*
      Run IBM Software Installer with catalog file found above.
      Place our /C qualifier last, so it will override any /C
         given by the user.
   */
   actionName = LEFT( actionName, 1 )
   SELECT
      WHEN actionName == 'C' THEN actionQualifier = ""
      OTHERWISE
         actionQualifier = "/A:" || actionName
   END

   sourceDrivePath || "OS2INST" userQualifiers "/C:" || catalogFile ,
      actionQualifier "/O:DRIVE /P:""" || productName || '"'

   /*
      If the Installer cannot be started, the error code will
      almost certainly be less than 0x800, the lowest CID-
      compatible error code.  These errors must be translated
      into something CID-compatible.  Higher (CID-compatible)
      errors come from the Installer and must be left alone.
   */
   IF ( rc > 0 ) & ( rc < 2048 ) THEN
      /*
         Substitute CID-compliant error code:
               Unexpected error -- cannot continue
      */
      rc = X2D( "1604" )
   RETURN rc



/*
   Check that an installation already exists upon which to operate.
   If yes, return the path to its root directory.
*/
CheckAlreadyInstalled:   PROCEDURE
   PARSE ARG which, actionNumber, messageFile

   path = GetFromRegistry( which )
   IF actionNumber > 1     /* CATALOG, INSTALL exempted */
   THEN
      DO
         IF path = "" THEN
            CALL NotThere messageFile
      END

   IF path \= "" & RIGHT(path, 1) \= "\" THEN
      path = path || "\"

   RETURN path



/*
   Check "err" for success or failure values as defined by CID spec.
   These are
   Success:    (err%256 = 0) | (err%256 = 254)
   Failure:    any other values
*/
CheckErr: PROCEDURE
   ARG err
   sev = err % 256;
   SELECT
      WHEN sev = 0     THEN RETURN 0
      WHEN sev = 254   THEN RETURN 0
      OTHERWISE RETURN 1
   END



/*
   Report missing installation and exit.
*/
NotThere:   PROCEDURE
   PARSE ARG messageFile

   "VIEW" messageFile """not installed"""
   /*
      Exit with CID-compliant errorlevel:
         Problem with product configuration / catalog file / package file
   */
   EXIT X2D( "0816" )


/*  H e l p   r o u t i n e s  */

GiveHelp:   PROCEDURE
   PARSE ARG messageFile

   "VIEW"  messageFile "command"

   EXIT 0
